//+------------------------------------------------------------------+
//|                                                      TestADX.mq5 |
//|                        Copyright 2017, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright   "Copyright 2009-2017, MetaQuotes Software Corp."
#property link        "http://www.mql5.com"
#property version     "1.00"

#define ADX_MAGIC 1234502
//---
#include <Trade\Trade.mqh>
#include <Trade\SymbolInfo.mqh>
#include <Trade\PositionInfo.mqh>
#include <Trade\AccountInfo.mqh>
//---
input int    InpADXPeriod     =14;  // ADX trend period
input double InpLots          =0.1; // Lots
input int    InpTakeProfit    =50;  // Take Profit (in pips)
input int    InpStopLoss      =20;  // Stop Loss (in pips)
input int    InpTrailingStop  =30;  // Trailing Stop Level (in pips)
input int    InpOpenLevel     =3;   // Open level (in pips)
input int    InpStopLevel     =10;  // Stop level (in pips)
//---
int ExtTimeOut=10; // time out in seconds between trade operations
int bar=50; // Bar Calculate
//+------------------------------------------------------------------+
//| ADX Sample expert class                                         |
//+------------------------------------------------------------------+
class CSampleExpert
  {
protected:
   double            m_adjusted_point;             // point value adjusted for 3 or 5 points
   CTrade            m_trade;                      // trading object
   CSymbolInfo       m_symbol;                     // symbol info object
   CPositionInfo     m_position;                   // trade position object
   CAccountInfo      m_account;                    // account info wrapper
   //--- indicators
   int               m_handle_adx;                // ADX indicator handle
   //--- indicator buffers
   double            m_buff_ADX_main[];           // ADX indicator main buffer
   double            m_buff_DI_p[];               // DI+ indicator buffer
   double            m_buff_DI_m[];               // DI- indicator buffer
   double            m_buff_close[];              // Close buffer
   double            m_buff_open[];               // Open buffer
   double            m_buff_high[];               // High buffer
   double            m_buff_low[];                // Low buffer
   datetime            m_buff_time[];                // Low buffer
   //--- indicator data for processing
   double            m_adx_up_level;
   double            m_adx_dn_level;
   double            m_close;
   double            m_open;
   double            m_open_level;
   double            m_stop_level;
   double            m_traling_stop;
   double            m_take_profit;
   double            m_stop_loss;
   datetime           time_up;
   datetime           time_dn;

public:
                     CSampleExpert(void);
                    ~CSampleExpert(void);
   bool              Init(void);
   void              Deinit(void);
   bool              Processing(void);
   double            UpLevel(void);
   double            DnLevel(void);

protected:
   bool              InitCheckParameters(const int digits_adjust);
   bool              InitIndicators(void);
   bool              LongModified(void);
   bool              ShortModified(void);
   bool              LongOpened(void);
   bool              ShortOpened(void);
  };
//--- global expert
CSampleExpert ExtExpert;
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double CSampleExpert::UpLevel(void)
{
double level=-1;
//---
for(int i=0;i<bar-1;i++)
{
if(m_buff_DI_p[i]<m_buff_DI_p[i+1] && m_buff_DI_m[i]<m_buff_DI_m[i+1])//&&m_buff_ADX_main[i]<m_buff_ADX_main[i+1]
if(m_buff_DI_m[i]<m_buff_DI_p[i]) {level=m_buff_high[i]+m_open_level;time_up=m_buff_time[i];break;}
}
//---
return(level);
}
//---
double CSampleExpert::DnLevel(void)
{
double level=-1;
//---
for(int i=0;i<bar-1;i++)
{
if(m_buff_DI_p[i]<m_buff_DI_p[i+1] && m_buff_DI_m[i]<m_buff_DI_m[i+1])//&&m_buff_ADX_main[i]<m_buff_ADX_main[i+1]
if(m_buff_DI_m[i]>m_buff_DI_p[i]) {level=m_buff_low[i]-m_open_level;time_dn=m_buff_time[i];break;}
}
//---
return(level);
}
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CSampleExpert::CSampleExpert(void) : m_adjusted_point(0),
                                     m_handle_adx(INVALID_HANDLE),
                                     m_adx_up_level(0),
                                     m_adx_dn_level(0),
                                     m_close(0),
                                     m_open(0),
                                     m_open_level(0),
                                     m_stop_level(0),
                                     m_traling_stop(0),
                                     m_take_profit(0),
                                     time_up(0),
                                     time_dn(0),
                                     m_stop_loss(0)
  {
   ArraySetAsSeries(m_buff_ADX_main,true);
   ArraySetAsSeries(m_buff_DI_p,true);
   ArraySetAsSeries(m_buff_DI_m,true);
   ArraySetAsSeries(m_buff_close,true);
   ArraySetAsSeries(m_buff_open,true);
   ArraySetAsSeries(m_buff_high,true);
   ArraySetAsSeries(m_buff_low,true);
   ArraySetAsSeries(m_buff_time,true);
  }
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CSampleExpert::~CSampleExpert(void)
  {
  }
//+------------------------------------------------------------------+
//| Initialization and checking for input parameters                 |
//+------------------------------------------------------------------+
bool CSampleExpert::Init(void)
  {
//--- initialize common information
   m_symbol.Name(Symbol());                  // symbol
   m_trade.SetExpertMagicNumber(ADX_MAGIC); // magic
   m_trade.SetMarginMode();
   m_trade.SetTypeFillingBySymbol(Symbol());
//--- tuning for 3 or 5 digits
   int digits_adjust=1;
   if(m_symbol.Digits()==3 || m_symbol.Digits()==5)
      digits_adjust=10;
   m_adjusted_point=m_symbol.Point()*digits_adjust;
//--- set default deviation for trading in adjusted points
   m_open_level   =InpOpenLevel*m_adjusted_point;
   m_stop_level  =InpStopLevel*m_adjusted_point;
   m_traling_stop =InpTrailingStop*m_adjusted_point;
   m_take_profit  =InpTakeProfit*m_adjusted_point;
   m_stop_loss  =InpStopLoss*m_adjusted_point;
//--- set default deviation for trading in adjusted points
   m_trade.SetDeviationInPoints(3*digits_adjust);
//---
   if(!InitCheckParameters(digits_adjust))
      return(false);
   if(!InitIndicators())
      return(false);
//--- succeed
   return(true);
  }
//+------------------------------------------------------------------+
//| Checking for input parameters                                    |
//+------------------------------------------------------------------+
bool CSampleExpert::InitCheckParameters(const int digits_adjust)
  {
//--- initial data checks
   if(InpTakeProfit*digits_adjust<m_symbol.StopsLevel())
     {
      printf("Take Profit must be greater than %d",m_symbol.StopsLevel());
      return(false);
     }
   if(InpTrailingStop*digits_adjust<m_symbol.StopsLevel())
     {
      printf("Trailing Stop must be greater than %d",m_symbol.StopsLevel());
      return(false);
     }
//--- check for right lots amount
   if(InpLots<m_symbol.LotsMin() || InpLots>m_symbol.LotsMax())
     {
      printf("Lots amount must be in the range from %f to %f",m_symbol.LotsMin(),m_symbol.LotsMax());
      return(false);
     }
   if(MathAbs(InpLots/m_symbol.LotsStep()-MathRound(InpLots/m_symbol.LotsStep()))>1.0E-10)
     {
      printf("Lots amount is not corresponding with lot step %f",m_symbol.LotsStep());
      return(false);
     }
//--- warning
   if(InpTakeProfit<=InpTrailingStop)
      printf("Warning: Trailing Stop must be less than Take Profit");
//--- succeed
   return(true);
  }
//+------------------------------------------------------------------+
//| Initialization of the indicators                                 |
//+------------------------------------------------------------------+
bool CSampleExpert::InitIndicators(void)
  {
//--- create ADX indicator
   if(m_handle_adx==INVALID_HANDLE)
      if((m_handle_adx=iADX(NULL,0,InpADXPeriod))==INVALID_HANDLE)
        {
         printf("Error creating ADX indicator");
         return(false);
        }
//--- succeed
   return(true);
  }
//+------------------------------------------------------------------+
//| Check for long position modifying                                |
//+------------------------------------------------------------------+
bool CSampleExpert::LongModified(void)
  {
   bool res=false;
//--- check for trailing stop
   if(InpTrailingStop>0)
     {
      if(m_symbol.Bid()-m_position.PriceOpen()>m_adjusted_point*InpTrailingStop)
        {
         double sl=NormalizeDouble(m_symbol.Bid()-m_traling_stop,m_symbol.Digits());
         double tp=m_position.TakeProfit();
         if(m_position.StopLoss()<sl || m_position.StopLoss()==0.0)
           {
            //--- modify position
            if(m_trade.PositionModify(Symbol(),sl,tp))
               printf("Long position by %s to be modified",Symbol());
            else
              {
               printf("Error modifying position by %s : '%s'",Symbol(),m_trade.ResultComment());
               printf("Modify parameters : SL=%f,TP=%f",sl,tp);
              }
            //--- modified and must exit from expert
            res=true;
           }
        }
     }
//--- result
   return(res);
  }
//+------------------------------------------------------------------+
//| Check for short position modifying                               |
//+------------------------------------------------------------------+
bool CSampleExpert::ShortModified(void)
  {
   bool   res=false;
//--- check for trailing stop
   if(InpTrailingStop>0)
     {
      if((m_position.PriceOpen()-m_symbol.Ask())>(m_adjusted_point*InpTrailingStop))
        {
         double sl=NormalizeDouble(m_symbol.Ask()+m_traling_stop,m_symbol.Digits());
         double tp=m_position.TakeProfit();
         if(m_position.StopLoss()>sl || m_position.StopLoss()==0.0)
           {
            //--- modify position
            if(m_trade.PositionModify(Symbol(),sl,tp))
               printf("Short position by %s to be modified",Symbol());
            else
              {
               printf("Error modifying position by %s : '%s'",Symbol(),m_trade.ResultComment());
               printf("Modify parameters : SL=%f,TP=%f",sl,tp);
              }
            //--- modified and must exit from expert
            res=true;
           }
        }
     }
//--- result
   return(res);
  }
//+------------------------------------------------------------------+
//| Check for long position opening                                  |
//+------------------------------------------------------------------+
bool CSampleExpert::LongOpened(void)
  {
   bool res=false;
//--- check for long position (BUY) possibility
   if(m_adx_up_level>0)
      if(m_close>m_open && m_close>m_adx_up_level && m_open<m_adx_up_level &&
      fabs(m_adx_up_level-m_close)<m_stop_level)
   //      if(MathAbs(m_adx_current)>(m_open_level) && m_DIm_current>m_DIm_previous)
           {
            double price=m_symbol.Ask();
            double tp   =m_symbol.Bid()+m_take_profit;
            double sl   =m_adx_dn_level;
            if(sl>m_adx_up_level)sl=m_symbol.Bid()-m_stop_loss;
            //--- check for free money
            if(m_account.FreeMarginCheck(Symbol(),ORDER_TYPE_BUY,InpLots,price)<0.0)
               printf("We have no money. Free Margin = %f",m_account.FreeMargin());
            else
              {
               //--- open position
               if(m_trade.PositionOpen(Symbol(),ORDER_TYPE_BUY,InpLots,price,sl,tp))
                  printf("Position by %s to be opened",Symbol());
               else
                 {
                  printf("Error opening BUY position by %s : '%s'",Symbol(),m_trade.ResultComment());
                  printf("Open parameters : price=%f,TP=%f",price,tp);
                 }
              }
            //--- in any case we must exit from expert
            res=true;
           }
//--- result
   return(res);
  }
//+------------------------------------------------------------------+
//| Check for short position opening                                 |
//+------------------------------------------------------------------+
bool CSampleExpert::ShortOpened(void)
  {
   bool res=false;
//--- check for short position (SELL) possibility
   if(m_adx_dn_level>0)
      if(m_close<m_open && m_close<m_adx_dn_level && m_open>m_adx_dn_level &&
      fabs(m_adx_dn_level-m_close)<m_stop_level)
   //      if(m_adx_current>(m_open_level) && m_DIm_current<m_DIm_previous)
           {
            double price=m_symbol.Bid();
            double tp   =m_symbol.Ask()-m_take_profit;
            double sl   =m_adx_up_level;//m_symbol.Ask()+m_stop_loss;
            if(sl<m_adx_dn_level)sl=m_symbol.Ask()+m_stop_loss;
            //--- check for free money
            if(m_account.FreeMarginCheck(Symbol(),ORDER_TYPE_SELL,InpLots,price)<0.0)
               printf("We have no money. Free Margin = %f",m_account.FreeMargin());
            else
              {
               //--- open position
               if(m_trade.PositionOpen(Symbol(),ORDER_TYPE_SELL,InpLots,price,sl,tp))
                  printf("Position by %s to be opened",Symbol());
               else
                 {
                  printf("Error opening SELL position by %s : '%s'",Symbol(),m_trade.ResultComment());
                  printf("Open parameters : price=%f,TP=%f",price,tp);
                 }
              }
            //--- in any case we must exit from expert
            res=true;
           }
//--- result
   return(res);
  }
//+------------------------------------------------------------------+
//| main function returns true if any position processed             |
//+------------------------------------------------------------------+
bool CSampleExpert::Processing(void)
  {
//--- refresh rates
   if(!m_symbol.RefreshRates())
      return(false);
//--- refresh indicators
   if(BarsCalculated(m_handle_adx)<bar)
      return(false);
   if(CopyBuffer(m_handle_adx,MAIN_LINE,1,bar,m_buff_ADX_main)!=bar || 
      CopyBuffer(m_handle_adx,PLUSDI_LINE,1,bar,m_buff_DI_p)!=bar || 
      CopyBuffer(m_handle_adx,MINUSDI_LINE,1,bar,m_buff_DI_m)!=bar||
      CopyHigh(NULL,0,1,bar,m_buff_high)!=bar||CopyLow(NULL,0,1,bar,m_buff_low)!=bar||
      CopyOpen(NULL,0,1,2,m_buff_open)!=2||CopyClose(NULL,0,1,2,m_buff_close)!=2)
      return(false);
      CopyTime(NULL,0,1,bar,m_buff_time);
//   m_indicators.Refresh();
//--- to simplify the coding and speed up access
//--- data are put into internal variables
   m_close         =m_buff_close[0];
   m_open          =m_buff_open[0];
   m_adx_up_level  =UpLevel();
   m_adx_dn_level  =DnLevel();
//--- it is important to enter the market correctly, 
//--- but it is more important to exit it correctly...   
//--- first check if position exists - try to select it
   if(m_position.Select(Symbol()))
     {
      if(m_position.PositionType()==POSITION_TYPE_BUY)
        {
         //--- try to close or modify long position
   //      if(LongClosed())
   //         return(true);
         if(LongModified())
            return(true);
        }
      else
        {
         //--- try to close or modify short position
    //     if(ShortClosed())
    //        return(true);
         if(ShortModified())
            return(true);
        }
     }
//--- no opened position identified
   else
     {
      //--- check for long position (BUY) possibility
      if(LongOpened())
         return(true);
      //--- check for short position (SELL) possibility
      if(ShortOpened())
         return(true);
     }
     Comment("Level Sell = ",m_adx_dn_level,"\n",
     "Time signal Sell - ",time_dn,"","\n","Level Buy = ",m_adx_up_level,"\n",
     "Time signal Buy - ",time_up,"\n","OpenLevel = ",m_open_level);
//--- exit without position processing
   return(false);
  }
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit(void)
  {
//--- create all necessary objects
   if(!ExtExpert.Init())
      return(INIT_FAILED);
//--- secceed
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert new tick handling function                                |
//+------------------------------------------------------------------+
void OnTick(void)
  {
   static datetime limit_time=0; // last trade processing time + timeout
//--- don't process if timeout
   if(TimeCurrent()>=limit_time)
     {
      //--- check for data
      if(Bars(Symbol(),Period())>bar+2*InpADXPeriod)
        {
         //--- change limit time by timeout in seconds if processed
         if(ExtExpert.Processing())
            limit_time=TimeCurrent()+ExtTimeOut;
        }
     }
  }
//+------------------------------------------------------------------+
